@@ -31,7 +31,7 @@ class SimultanObject(object):
     @classproperty
     def _cls_instances_dict(cls):
         if cls._cls_instances_dict_cache is None:
-            cls._cls_instances_dict_cache = dict(zip([x.id for x in cls._cls_instances], [x() for x in cls._cls_instances]))
+            cls._cls_instances_dict_cache = dict(zip([x.id for x in cls._cls_instances], [x for x in cls._cls_instances]))
         return cls._cls_instances_dict_cache
 
     @classproperty
@@ -181,7 +181,10 @@ class SimultanObject(object):
         Returns the slot extension of the component
         :param
         """
-        return int(re.findall("[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?", self.current_slot)[-1])
+        try:
+            return int(re.findall("[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?", self.current_slot)[-1])
+        except Exception as e:
+            return None
 
 
 class List(UserList):
@@ -240,6 +243,58 @@ class List(UserList):
             self._wrapped_obj.Name = value
 
 
+class ReferenceList(UserList):
+
+    def __init__(self, *args, **kwargs):
+        self._wrapped_obj = kwargs.get('wrapped_obj', None)
+        if self._wrapped_obj is not None:
+            self.data = self._wrapped_obj.ReferencedComponents.Items
+        self._contained_components = kwargs.get('contained_components', None)
+        self._contained_parameters = kwargs.get('contained_parameters', None)
+        self._template_parser = kwargs.get('template_parser', None)
+        self._data_model_id = kwargs.get('data_model_id', None)
+
+    def __getitem__(self, i):
+        if isinstance(i, slice):
+            if self._template_parser is None:
+                return self.__class__(self.data[i])
+            return [self._template_parser.create_python_object(x) for x in self.__class__(self.data[i])]
+        else:
+            if self._template_parser is None:
+                return self.data[i]
+            return self._template_parser.create_python_object(self.data[i])
+
+    def __repr__(self):
+        return f'{self.name}: ' + repr(list(self.data))
+
+    @property
+    def contained_components(self):
+        if self._contained_components is None:
+            if self._wrapped_obj is not None:
+                self._contained_components = [self._template_parser.create_python_object(x) for x in
+                                              self._wrapped_obj.ContainedComponentsAsList]
+        return self._contained_components
+
+    @property
+    def contained_parameters(self):
+        if self._contained_parameters is None:
+            if self._wrapped_obj is not None:
+                self._contained_parameters = {x.Name: x.get_ValueCurrent() for x in
+                                              self._wrapped_obj.ContainedParameters.Items}
+        return self._contained_parameters
+
+    @property
+    def name(self):
+        if self._wrapped_obj is not None:
+            if hasattr(self._wrapped_obj, 'Name'):
+                return self._wrapped_obj.Name
+
+    @name.setter
+    def name(self, value):
+        if self._wrapped_obj is not None:
+            self._wrapped_obj.Name = value
+
+
 class ValueField(pd.DataFrame):
 
     _metadata = pd.DataFrame._metadata + ['_wrapped_obj', '_template_parser', '_data_model_id']
@@ -320,24 +375,31 @@ class BuildInFace(SimultanObject):
         return self._template_parser.create_python_object(obj)
 
     def get_geo_instances(self):
-        geo_instances = []
+        try:
+            geo_instances = []
 
-        for item in self._wrapped_obj.GeometryInstances.Items:
+            for item in self._wrapped_obj.GeometryInstances.Items:
 
-            geom_placement = next(x for x in item.Placements.Items if isinstance(x, InstancePlacementGeometry))
+                if item.Placements.Items.__len__() == 0:
+                    continue
 
-            file_id = geom_placement.FileId
-            geometry_id = geom_placement.GeometryId
+                geom_placement = next(x for x in item.Placements.Items if isinstance(x, InstancePlacementGeometry))
 
-            # geo_model = self._template_parser.data_models[self._data_model_id].get_typed_model_by_file_id(file_id)
-            geo_model = self._template_parser.typed_geo_models[file_id]
-            geo_instance = geo_model.get_face_by_id(geometry_id)
+                file_id = geom_placement.FileId
+                geometry_id = geom_placement.GeometryId
 
-            # geo_instance = self._template_parser.data_models[self._data_model_id].typed_geo_models[file_id].get_face_by_id(geometry_id)
+                # geo_model = self._template_parser.data_models[self._data_model_id].get_typed_model_by_file_id(file_id)
+                geo_model = self._template_parser.typed_geo_models[file_id]
+                geo_instance = geo_model.get_face_by_id(geometry_id)
 
-            geo_instances.append(geo_instance)
+                # geo_instance = self._template_parser.data_models[self._data_model_id].typed_geo_models[file_id].get_face_by_id(geometry_id)
 
-        return geo_instances
+                geo_instances.append(geo_instance)
+
+            return geo_instances
+        except Exception as e:
+            logger.error(f'BuildInFace {self.name}; {self.id}: Error while getting geo instances: {e}')
+            return []
 
 
 class BuildInVolume(SimultanObject):
